{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
OSCPM-ASPEL module for the osCommerce Product Manager for Windows
(oscpm1_aspel). Copyright 2004 by Mario A. Valdez-Ramirez.

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DB, KDaoTable, KDaoDataBase, IdBaseComponent,
  IdComponent, IdTCPServer, Gauges, Grids, DBGrids;

CONST
  o1as_AppName = 'Sincronizador de OSCPM1 - ASPEL';
  o1as_AppShortName = 'oscpm1-aspel-sync';
  o1as_Version = '0.0.1';
  o1as_Homepage = 'http://www.mariovaldez.net/software/oscpmwin/modules/';
  o1as_Copyright = 'Copyright 2003, 2004 by Mario A. Valdez-Ramirez';
  o1as_Email = 'mario@mariovaldez.org';
  o1as_TCPPort = 27572;
  o1as_TCPHost = '127.0.0.1';
  o1as_TCPTimeout = 30000;
  o1as_TCPMaxChars = 250;
  o1as_MaxSimConnections = 10; 
  o1as_QuitCommand = 'QUIT OSCPM1';
  o1as_StatCommand = 'STATS OSCPM1';
  o1as_OKCode = 'OK';
  o1as_StatCode = 'STAT';
  o1as_StatCodeEnd = 'STAT END';
  o1as_ModNACode = 'NOT FOUND';
  o1as_ModErrorCode = 'ERROR ERROR';
  o1as_GaugeBucket = 25;
  o1as_StatTotalReqs = 'Consultas totales';
  o1as_StatMatchingReqs = 'Consultas coincidentes';
  o1as_StatNoMatchReqs = 'Consultas sin coincidentes';
  o1as_StatErrorReqs = 'Consultas con error';


TYPE
  To1as_Form_Main = class(TForm)
    o1as_Database_Main: TKADaoDatabase;
    o1as_DAOTable_Main: TKADaoTable;
    o1as_GroupBox_Main: TGroupBox;
    o1as_label_conninfo: TLabel;
    o1as_label_recinfo: TLabel;
    o1as_label_dbinfo: TLabel;
    o1as_TCPServer_Main: TIdTCPServer;
    o1as_label_connstat: TLabel;
    o1as_GroupBox_Stats: TGroupBox;
    o1as_label_totalreqs: TLabel;
    o1as_label_totalreqsLeg: TLabel;
    o1as_label_recfoundLeg: TLabel;
    o1as_label_recfound: TLabel;
    o1as_label_recnotfound: TLabel;
    o1as_label_recnotfoundLeg: TLabel;
    o1as_label_recerrorLeg: TLabel;
    o1as_label_recerror: TLabel;
    o1as_label_dbinfo2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure o1as_TCPServer_MainExecute(AThread: TIdPeerThread);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    o1as_RunningLinks : LONGINT;
  public
    PROCEDURE o1as_PRMakeConsult;
    FUNCTION o1as_FNGetProductItem (IndexData : STRING) : STRING;
    PROCEDURE o1as_PRShowConnStatRec;
    PROCEDURE o1as_PRShowConnStatFin;
    PROCEDURE o1as_PRCloseAll (Sender: TObject);
  end;

  o1as_QueryCounterRecord = RECORD
                              ThreadID : CARDINAL;
                              TotalReqs,
                              RecNotFound,
                              RecFound,
                              RecError : LONGINT;
                            END;
  o1as_QueryCounterList = ARRAY [0..o1as_MaxSimConnections] OF o1as_QueryCounterRecord;


VAR
  o1as_Form_Main: To1as_Form_Main;
  o1as_AspelDBFile : STRING;
  o1as_AspelDBTable : STRING;
  o1as_AspelInputField : STRING;
  o1as_AspelOutputField : STRING;
  o1as_AspelTCPPort : LONGINT;
  o1as_CurrInputItem : STRING;
  o1as_CurrOutputItem : STRING;
  o1as_CurrExtraItem : STRING;
  o1as_ClientIP : STRING;
  o1as_AspelShowAll : BOOLEAN;
  o1as_QueryCounters : o1as_QueryCounterList;

  
implementation

{$R *.dfm}

uses IdSocketHandle;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.FormCreate (Sender: TObject);
VAR
  ServerBind : TIdSocketHandle;
BEGIN
  o1as_RunningLinks := 0;
  o1as_Database_Main.ReadOnly := TRUE;
  o1as_DAOTable_Main.ReadOnly := TRUE;
  o1as_AspelDBFile := TRIM (ParamStr(1));
  o1as_AspelDBTable := TRIM (ParamStr (2));
  o1as_AspelInputField := TRIM (ParamStr(3));
  o1as_AspelOutputField := TRIM (ParamStr(4));
  IF ((o1as_AspelDBFile <> '') AND
      (o1as_AspelDBTable <> '') AND
      (o1as_AspelInputField <> '') AND
      (o1as_AspelOutputField <> '')) THEN
    TRY
      o1as_Database_Main.Database := o1as_AspelDBFile;
      o1as_Database_Main.Connected := TRUE;
    EXCEPT
      MessageBeep (MB_ICONEXCLAMATION);
      MessageDlg ('Hubo un error al abrir la base de datos de ASPEL ' + o1as_AspelDBFile + '. Terminando.', mtError, [mbAbort], 0);
      Application.Terminate;
    END
  ELSE
    BEGIN
      MessageBeep (MB_ICONEXCLAMATION);
      MessageDlg ('Parmetros insuficientes o invlidos. Terminando.'#13#10 +
                  'archivoMDB tabla campoindice campobuscado [puertoTCP [panel]] ', mtError, [mbAbort], 0);
      Application.Terminate;
    END;
  o1as_AspelTCPPort := STRTOINTDEF (TRIM (ParamStr (5)), o1as_TCPPort);
  o1as_TCPServer_Main.Bindings.Clear;
  ServerBind := o1as_TCPServer_Main.Bindings.Add;
  ServerBind.Port := o1as_AspelTCPPort;
  ServerBind.IP := o1as_TCPHost;
  TRY
    o1as_TCPServer_Main.Active := TRUE;
  EXCEPT
    MessageBeep (MB_ICONEXCLAMATION);
    MessageDlg ('No se puede establecer el enlace ' + o1as_TCPHost + ':' + INTTOSTR (o1as_AspelTCPPort) + ' con la aplicacin principal. Terminando.', mtError, [mbAbort], 0);
    Application.Terminate;
  END;
  o1as_AspelShowAll := (TRIM (ParamStr (6)) <> '');
  SetPriorityClass (GetModuleHandle (NIL), NORMAL_PRIORITY_CLASS);
  SetThreadPriority (GetCurrentThread, THREAD_PRIORITY_LOWEST);
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.FormShow(Sender: TObject);
begin
  o1as_Form_Main.Caption := o1as_AppName + ' ' + o1as_Version;
  o1as_GroupBox_Main.Caption := '';
  o1as_GroupBox_Stats.Caption := '';
  o1as_label_dbinfo.Caption := o1as_AspelDBFile;
  o1as_label_dbinfo2.Caption := o1as_AspelInputField + ' -> ' + o1as_AspelOutputField + ' @' + o1as_AspelDBTable;
  o1as_label_conninfo.Caption := 'Esperando conexiones en ' + o1as_TCPHost + ':' + INTTOSTR (o1as_AspelTCPPort);
  o1as_label_recinfo.Caption := '';
  o1as_label_connstat.Caption := 'Esperando conexiones.';
  o1as_label_totalreqs.Caption := '0';
  o1as_label_recfound.Caption := '0';
  o1as_label_recnotfound.Caption := '0';
  o1as_label_recerror.Caption := '0';
  o1as_GroupBox_Main.Visible := o1as_AspelShowAll;
  IF (o1as_AspelShowAll = TRUE) THEN o1as_GroupBox_Stats.Top := (o1as_GroupBox_Main.Height - 1);
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.o1as_TCPServer_MainExecute(AThread: TIdPeerThread);
begin
  IF (o1as_RunningLinks < o1as_MaxSimConnections) THEN
    BEGIN
      INC (o1as_RunningLinks);
      o1as_QueryCounters[o1as_RunningLinks].ThreadID := AThread.ThreadID;
      AThread.OnTerminate := o1as_PRCloseAll;
      o1as_CurrInputItem := '';
      o1as_PRShowConnStatRec;
      TRY
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_AppName + ' ' + o1as_Version);
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_Homepage);
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_Copyright);
        AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_Email);
        REPEAT
          o1as_CurrInputItem := AThread.Connection.ReadLn (#13#10, o1as_TCPTimeout, o1as_TCPMaxChars);
          o1as_CurrInputItem := TRIM (AdjustLineBreaks (o1as_CurrInputItem));
          IF (UPPERCASE (o1as_CurrInputItem) = o1as_QuitCommand) THEN
            BEGIN
              AThread.Connection.Disconnect;
              BREAK;
            END
          ELSE IF (UPPERCASE (o1as_CurrInputItem) = o1as_StatCommand) THEN
            BEGIN
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatTotalReqs + ': ' + INTTOSTR (o1as_QueryCounters[0].TotalReqs));
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatMatchingReqs + ': ' + INTTOSTR (o1as_QueryCounters[0].RecFound));
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatNoMatchReqs + ': ' + INTTOSTR (o1as_QueryCounters[0].RecNotFound));
              AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_StatErrorReqs + ': ' + INTTOSTR (o1as_QueryCounters[0].RecError));
              AThread.Connection.WriteLn (o1as_StatCodeEnd);
            END
          ELSE
            BEGIN
              AThread.Synchronize (o1as_PRMakeConsult);
              AThread.Connection.WriteLn (o1as_OKCode + ' ' + o1as_CurrOutputItem);
              IF (o1as_AspelShowAll) THEN AThread.Connection.WriteLn (o1as_StatCode + ' ' + o1as_CurrExtraItem);
            END;
        UNTIL ((UPPERCASE (o1as_CurrInputItem) = o1as_QuitCommand) OR (o1as_CurrInputItem = ''));
      FINALLY
        AThread.connection.Disconnect;
      END;
      o1as_PRShowConnStatFin;
    END
  ELSE AThread.Connection.Disconnect;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRMakeConsult;
BEGIN
  Application.ProcessMessages;
  INC (o1as_QueryCounters[0].TotalReqs);
  IF (o1as_CurrInputItem <> '') THEN
    BEGIN
      o1as_CurrOutputItem := o1as_FNGetProductItem (o1as_CurrInputItem);
      o1as_CurrExtraItem := o1as_CurrInputItem + ' <-> ' + o1as_CurrOutputItem;
    END
  ELSE
    BEGIN
      o1as_CurrOutputItem := o1as_ModNACode;
      o1as_CurrExtraItem := o1as_CurrInputItem + ' <-> ' + o1as_ModNACode;
    END;
  o1as_label_recinfo.Caption := o1as_CurrInputItem + ' <-> ' + o1as_CurrOutputItem;
  o1as_label_totalreqs.Caption := INTTOSTR (o1as_QueryCounters[0].TotalReqs);
  o1as_label_recfound.Caption := INTTOSTR (o1as_QueryCounters[0].RecFound);
  o1as_label_recnotfound.Caption := INTTOSTR (o1as_QueryCounters[0].RecNotFound);
  o1as_label_recerror.Caption := INTTOSTR (o1as_QueryCounters[0].RecError);
  o1as_label_connstat.Caption := 'Conexiones totales ' + INTTOSTR (o1as_RunningLinks);
  Application.ProcessMessages;
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRShowConnStatRec;
BEGIN
  o1as_label_connstat.Caption := 'Recibiendo conexin ' + INTTOSTR (o1as_RunningLinks);
END;


{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRShowConnStatFin;
BEGIN
  o1as_label_connstat.Caption := 'Conexin terminada ' + INTTOSTR (o1as_RunningLinks);
  o1as_label_recinfo.Caption := '';
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
FUNCTION To1as_Form_Main.o1as_FNGetProductItem (IndexData : STRING) : STRING;
BEGIN
  o1as_DAOTable_Main.Active := FALSE;
  o1as_DAOTable_Main.SQL.Clear;
  o1as_DAOTable_Main.SQL.Add ('select ' + o1as_AspelInputField + ', ' + o1as_AspelOutputField + ' from ' + o1as_AspelDBTable + ' where ' + o1as_AspelInputField + '="' + IndexData + '";');
  TRY
    o1as_DAOTable_Main.Active := TRUE;
    o1as_DAOTable_Main.First;
    IF (NOT o1as_DAOTable_Main.EOF) THEN
      BEGIN
        o1as_FNGetProductItem := o1as_DAOTable_Main.Fields[1].AsString;
        INC (o1as_QueryCounters[0].RecFound);
      END
    ELSE
      BEGIN
        o1as_FNGetProductItem := o1as_ModNACode;
        INC (o1as_QueryCounters[0].RecNotFound);
      END;
  EXCEPT
    o1as_FNGetProductItem := o1as_ModErrorCode;
    INC (o1as_QueryCounters[0].RecError);
  END;
END;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
procedure To1as_Form_Main.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  IF (o1as_RunningLinks = 0) THEN Action := caFree ELSE Action := caNone;
end;



{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
{%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
PROCEDURE To1as_Form_Main.o1as_PRCloseAll (Sender: TObject);
BEGIN
  DEC (o1as_RunningLinks);
  IF (o1as_RunningLinks = 0) THEN Application.Terminate;
END;






END.
